#!/usr/bin/env python3.7
'''
use this to get all images on your local host, after you did manager.py getAllImages
'''
import sqlite3
import urllib.request
import urllib.error
import urllib.parse
import os
import sys
import json
from concurrent.futures import ThreadPoolExecutor

THREADS = 12


def get(e):
    url, local = e
    directory = "/".join(local.split("/")[:-1])

    if not os.path.isdir(directory):
        os.makedirs(directory, exist_ok=True)

    if not os.path.isfile(local):
        try:
            print(f"Downloading {url} to {local}")
            urllib.request.urlretrieve(url, local)
        except (UnicodeEncodeError, urllib.error.URLError):
            sys.stderr.write('\n\t\t\t\t\tfail {0}'.format(url))
            pass
        except Exception as e:
            pass


class PlantNet:
    def __init__(self):
        self.dh = sqlite3.connect("database-full.db")
        self.cursor = self.dh.cursor()
        self.groups = ['useful', 'the-plant-list', 'weurope', 'invasion', 'weeds']
        self.executor = ThreadPoolExecutor(THREADS)

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self.executor.shutdown()
        self.dh.close()

    def get_list(self, topic):
        url = f"https://api.plantnet.org/v1/projects/{topic}/species?lang=en&pageSize=0&illustratedOnly=true"
        d = urllib.request.urlopen(url)
        data_ = d.read().decode(d.headers.get_content_charset())
        return json.loads(str(data_))

    def check_local_db(self, name):
        self.cursor.execute("SELECT name_lat, id plant_id FROM names WHERE lower(name_lat)=? UNION SELECT name_lat, plant_id FROM names_syn WHERE lower(name_lat)=?", [name.lower(), name.lower()])
        d = self.cursor.fetchone()
        if d:
            return d[1]
        return None

    def plantnet_images(self, name, group):
        output = []
        urli = u"https://api.plantnet.org/v1/projects/"+group+"/species/"+urllib.parse.quote(name)
        di = urllib.request.urlopen(urli)
        datai = json.loads(di.read().decode(di.headers.get_content_charset()))
        for it in datai['images'].keys():
            for img in datai['images'][it]:
                url = img['o'] + ".jpg"
                output.append(url)
                if len(output) > 1000:
                    return output
        return output

    def run(self):
        for group in self.groups:
            for e in self.get_list(group):
                name_plantnet = e.get('name')
                author = e.get('author')
                plant_id = self.check_local_db(name_plantnet)
                if plant_id and author and name_plantnet:
                    for url in self.plantnet_images(name_plantnet+" "+author, group):
                        path = os.path.join(str(plant_id), os.path.basename(url))
                        if not os.path.isfile(path):
                            self.executor.submit(get, (url, path))

def download():
    with ThreadPoolExecutor(THREADS) as p:
        with sqlite3.connect('database-full.db') as rv_a:
            cur = rv_a.cursor().execute("SELECT url, path FROM images")
            for i, row in enumerate(cur.fetchall()):
                if not os.path.isfile(row[1]):
                    p.submit(get, (row[0], row[1]))
                    sys.stdout.write('\r loaded {0} from DB to queue'.format(i))
                else:
                    cur.execute("DELETE FROM images WHERE path=?", [row[1]])
                    rv_a.commit()


download()
with PlantNet() as p:
    p.run()
